unit Matemat;

{$N+,G+,F+,O+,E-}

interface

const
    Maxmat=16;

type
    TMatrix = array [1..Maxmat] of array [1..Maxmat] of Real;
    TLine = set of 1..Maxmat;
    TStrk = 1..Maxmat;

function DetMat(N: TStrk; var A): Real;
procedure AddMat(N, M: TStrk; var A, B);
procedure MultMat(M, N, L: TStrk; var A, B, C);

function Sh(A: Real): Real;
function Ch(A: Real): Real;

type
  Complex = record
    Re, Im: Real;
  end;

procedure AddC   (A, B: Complex; var C: Complex);
procedure SubC   (A, B: Complex; var C: Complex);
procedure MultC  (A, B: Complex; var C: Complex);
procedure DelC   (A, B: Complex; var C: Complex);
procedure SinC   (A: Complex; var C: Complex);
procedure CosC   (A: Complex; var C: Complex);
procedure TnC    (A: Complex; var C: Complex);
procedure ShC    (A: Complex; var C: Complex);
procedure ChC    (A: Complex; var C: Complex);
procedure ThC    (A: Complex; var C: Complex);
procedure LnC    (A: Complex; var C: Complex);
procedure ExpC   (A: Complex; var C: Complex);
procedure AtnC   (A: Complex; var C: Complex);
procedure AthC   (A: Complex; var C: Complex);
procedure SqrC   (A: Complex; var C: Complex);
procedure SqrtC  (A: Complex; var C: Complex);
procedure WriteC (A: Complex);

type
  TFunc = function(X: Real): Extended;

function Integral(A, B, T: Extended; Func: TFunc): Extended;
(*
const
  CodeArray: array [1..120] of array [1..4] of Char =
 ('0   ','1   ','2   ','3   ','4   ','5   ','6   ','7   ','8   ','9   ',
  'E   ','/-/ ','.   ','BS  ','+   ','-   ','*   ','/   ','X^2 ','X  ',
  'NOP ','[X] ','{X} ','X^Y ','Exp ','10^X','Ln  ','Log ','DIV ','MOD ',
  'Sin ','Cos ','Tan ','ASin','ACos','ATan','Sh  ','Ch  ','Th  ','ASh ',
  'ACh ','ATh ','Rnd ','Roun','And ','Not ','Or  ','Xor ','D'#26'R ','R'#26'D ',
  'XY ','   ','St  ','S  ','ClrX','Prog','Goto','Mess','Gosb','Ret ',
  'Stop','CMem','For ','EOF ',''#26'"','"'#26'','Max ','Min ','    ','    ',
  'X'#26'M ','M'#26'X ','M+X ','X+M ','M-X ','X-M ','M*X ','X*M ','M/X ','X/M ',
  'KX'#26'M','KM'#26'X','KM+X','KX+M','KM-X','KX-M','KM*X','KX*M','KM/X','KX/M',
  'X<Y ','X>Y ','X=Y ','X<0 ','X>0 ','X=0 ','K<  ','K>  ','K=  ','K<0 ',
  'K>0 ','K=0 ','X<M ','X>M ','X=M ','M<0 ','M>0 ','M=0 ','XM ','YM ',
  'Lp0 ','Lp1 ','Lp2 ','Lp3 ','Lp4 ','Lp5 ','Lp6 ','Lp7 ','Lp8 ','Lp9 ');
*)
implementation

procedure AddMat(N, M: TStrk; var A, B);

var
  X: array [1..Maxmat*Maxmat] of Real absolute A;
  Y: array [1..Maxmat*Maxmat] of Real absolute B;
  I: byte;

begin
for I := 1 to 2*N*M do
  X[I] := X[I] + Y[I];
end;

procedure MultMat(M, N, L: TStrk; var A, B, C);

var
  X: array [1..Maxmat*Maxmat] of Real absolute A;
  Y: array [1..Maxmat*Maxmat] of Real absolute B;
  Z: array [1..Maxmat*Maxmat] of Real absolute C;
  I, J, K: Byte;
  Summ: Real;

begin
  Summ := 0;
  for I := 1 to L do
    for j := 1 to N do
    begin
      for K := 1 to M do
        Summ := Summ + X[J+(K-1)*N]*Y[(I-1)*M+K];
      Z[(i-1)*N+J] := Summ;
      Summ := 0;
    end;
end;

function DetCalc(var M: TMatrix; L: TStrk; S: TLine): Real;

function Stepen(I: byte): Integer;
begin
  Stepen := 1-2*(I mod 2);
end;

var
  I, K: byte;
  Summ: real;

begin
  K := 1;
  Summ := 0;
  if L = Maxmat then
    for I := 1 to Maxmat do
      if not (I in S) then
      begin
        DetCalc := M[L, I];
        Exit;
      end;
  for I := 1 to Maxmat do
    if not (I in S) then
    begin
      if M[L,I] <> 0 then
        Summ := Summ + Stepen(K+1)*DetCalc(M, L+1, S+[I])*M[L, I];
      K := K+1;
    end;
  DetCalc := Summ;
end;

function DetMat(N: TStrk; var A)        :Real;

var
    Mat: TMatrix;
    Z: array [1..Maxmat*Maxmat] of Real absolute A;
    I, J: byte;
    P: TLine;

begin
  for I := 1 to N do
    for J := 1 to N do
      Mat[Maxmat-N+I, Maxmat-N+J] := Z[(I-1)*N+J];
  P := [];
  for I := 1 to Maxmat-N do P := P+[I];
  DetMat := DetCalc(Mat, Maxmat-N+1, P);
end;

function Sh;
begin
  Sh := (Exp(A)-1/Exp(A))/2;
end;

function Ch;
begin
  Ch := (Exp(A)+1/Exp(A))/2;
end;

procedure AddC;
begin
  C.Re := A.Re+B.Re;
  C.Im := A.Im+B.Im;
end;

procedure SubC;
begin
  C.Re := A.Re-B.Re;
  C.Im := A.Im-B.Im;
end;

procedure MultC;
begin
  C.Re := A.Re*B.Re-A.Im*B.Im;
  C.Im := A.Re*B.Im+A.Im*B.Re;
end;

procedure DelC;
begin
  C.Re := (A.Re*B.Re-A.Im*B.Im)/(sqr(B.Re)+sqr(B.Im));
  C.Im := (A.Re*B.Im+A.Im*B.Re)/(sqr(B.Re)+sqr(B.Im));
end;

procedure WriteC;
begin
  write(A.Re);
  if A.Im<>0 then
  begin
    if A.Im>0 then write(' +') else write(' -');
    write(abs(A.Im),' i');
  end;
end;

procedure SinC;
begin
  C.Re := sin(A.Re)*ch(A.Im);
  C.Im := cos(A.Re)*sh(A.Im);
end;

procedure CosC;
begin
  C.Re := cos(A.Re)*ch(A.Im);
  C.Im := -sin(A.Re)*sh(A.Im);
end;

procedure TnC;
begin
  C.Re := sin(2*A.Re)/(cos(2*A.Re)+ch(2*A.Im));
  C.Im := sh(2*A.Im)/(cos(2*A.Re)+ch(2*A.Im));
end;

procedure LnC;
begin
  C.Re := 0.5*ln(sqr(A.Re)+sqr(A.Im));
  C.Im := arctan(A.Im/A.Re);
end;

procedure ExpC;
begin
  C.Re := exp(A.Re)*cos(A.Im);
  C.Im := exp(A.Re)*sin(A.Im);
end;

procedure ShC;
begin
  C.Re := sh(A.Re)*cos(A.Im);
  C.Im := ch(A.Re)*sin(A.Im);
end;

procedure ChC;
begin
  C.Re := ch(A.Re)*cos(A.Im);
  C.Im := sh(A.Re)*sin(A.Im);
end;

procedure ThC;
begin
  C.Re := sh(2*A.Re)/(cos(2*A.Im)+ch(2*A.Re));
  C.Im := sin(2*A.Im)/(cos(2*A.Im)+ch(2*A.Re));
end;

procedure AtnC;
begin
  C.Re := 0.5*arctan(2*A.Re/(1-sqr(A.Re)-sqr(A.Im)));
  C.Im := 0.25*ln((sqr(A.Re)+sqr(A.Im+1))/(sqr(A.Re)+sqr(A.Im-1)));
end;

procedure AthC;
var
  X, Y: Complex;
begin
  X.Re := -A.Im;
  X.Im := A.Re;
  AtnC(x, y);
  C.Re := Y.Im;
  C.Im := -Y.Re;
end;

procedure SqrC;
begin
  C.Re := sqr(A.Re)-sqr(A.Im);
  C.Im := 2*A.Re*A.Im;
end;

procedure SqrtC;
begin
  C.Re := sqrt((A.Re+sqrt(sqr(A.Re)+sqr(A.Im)))/2);
  C.Im := sqrt((-A.Re+sqrt(sqr(A.Re)+sqr(A.Im)))/2);
end;

function Integral(A, B, T: Extended; Func: TFunc): Extended;
var
  Summ, H, X: Extended;
  I, Step: Longint;
begin
  Step := Round(1/T);
  H := (B-A)/Step;
  Summ := 0;
  X := A+H/2;
  while X < B do
  begin
    Summ := Summ+(Func(X));
    X := X+H;
  end;
  Integral := Summ*H;
end;

end.
